"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInstructionFunctionLowLevelFragment = void 0;
const nodes_1 = require("../../../nodes");
const shared_1 = require("../../../shared");
const ImportMap_1 = require("../ImportMap");
const common_1 = require("./common");
const instructionAccountTypeParam_1 = require("./instructionAccountTypeParam");
function getInstructionFunctionLowLevelFragment(scope) {
    const { instructionNode, programNode, dataArgsManifest, nameApi, customInstructionData, } = scope;
    const imports = new ImportMap_1.ImportMap();
    const hasAccounts = instructionNode.accounts.length > 0;
    const hasLegacyOptionalAccounts = instructionNode.optionalAccountStrategy === 'omitted' &&
        instructionNode.accounts.some((account) => account.isOptional);
    const customData = customInstructionData.get(instructionNode.name);
    const hasData = !!customData || instructionNode.arguments.length > 0;
    const hasArgs = !!customData ||
        instructionNode.arguments.filter((field) => !field.defaultValue || field.defaultValueStrategy !== 'omitted').length > 0;
    const instructionDataName = nameApi.instructionDataType(instructionNode.name);
    const argsType = customData
        ? dataArgsManifest.looseType.render
        : nameApi.dataArgsType(instructionDataName);
    const encoderFunction = customData
        ? dataArgsManifest.encoder.render
        : `${nameApi.encoderFunction(instructionDataName)}()`;
    if (customData) {
        imports.mergeWith(dataArgsManifest.looseType, dataArgsManifest.encoder);
    }
    const accountTypeParamsFragment = (0, common_1.mergeFragments)(instructionNode.accounts.map((account) => (0, instructionAccountTypeParam_1.getInstructionAccountTypeParamFragment)({
        ...scope,
        instructionAccountNode: account,
        allowAccountMeta: true,
    })), (renders) => renders.join(', '));
    const accounts = instructionNode.accounts.map((account) => {
        const typeParam = `TAccount${(0, shared_1.pascalCase)(account.name)}`;
        const defaultValueFragment = getDefaultValue(account, programNode, nameApi, instructionNode.optionalAccountStrategy === 'omitted');
        imports.mergeWith(defaultValueFragment);
        const defaultValue = defaultValueFragment.render || null;
        const isOptionalOrHasLowLevelDefaultValues = account.isOptional || defaultValue !== null;
        return {
            ...account,
            typeParam,
            defaultRole: getDefaultRole(account),
            defaultValue,
            isOptionalOrHasLowLevelDefaultValues,
        };
    });
    return (0, common_1.fragmentFromTemplate)('instructionFunctionLowLevel.njk', {
        instruction: instructionNode,
        program: programNode,
        functionName: nameApi.instructionRawFunction(instructionNode.name),
        instructionType: nameApi.instructionType(instructionNode.name),
        hasAccounts,
        hasLegacyOptionalAccounts,
        accounts,
        hasData,
        hasArgs,
        argsType,
        encoderFunction,
        accountTypeParams: accountTypeParamsFragment.render,
    })
        .mergeImportsWith(imports, accountTypeParamsFragment)
        .addImports('solanaAddresses', ['Address'])
        .addImports('solanaInstructions', ['IAccountMeta'])
        .addImports('shared', hasAccounts ? ['accountMetaWithDefault'] : [])
        .addImports('solanaInstructions', hasAccounts ? ['AccountRole'] : []);
}
exports.getInstructionFunctionLowLevelFragment = getInstructionFunctionLowLevelFragment;
function getDefaultRole(account) {
    if (account.isSigner === true && account.isWritable) {
        return 'AccountRole.WRITABLE_SIGNER';
    }
    if (account.isSigner === true) {
        return 'AccountRole.READONLY_SIGNER';
    }
    if (account.isWritable) {
        return 'AccountRole.WRITABLE';
    }
    return 'AccountRole.READONLY';
}
function getDefaultValue(account, program, nameApi, usesLegacyOptionalAccounts) {
    var _a, _b;
    if (account.isOptional && usesLegacyOptionalAccounts) {
        return (0, common_1.fragment)('');
    }
    const defaultValue = (_a = account.defaultValue) !== null && _a !== void 0 ? _a : null;
    if (account.isOptional || (0, nodes_1.isNode)(defaultValue, 'programIdValueNode')) {
        return (0, common_1.fragment)(`{ address: "${program.publicKey}" as Address<"${program.publicKey}">, role: AccountRole.READONLY }`);
    }
    if ((0, nodes_1.isNode)(defaultValue, 'publicKeyValueNode')) {
        return (0, common_1.fragment)(`"${defaultValue.publicKey}" as Address<"${defaultValue.publicKey}">`);
    }
    if ((0, nodes_1.isNode)(defaultValue, 'programLinkNode')) {
        const programAddress = nameApi.programAddressConstant(defaultValue.name);
        const importFrom = (_b = defaultValue.importFrom) !== null && _b !== void 0 ? _b : 'generatedPrograms';
        return (0, common_1.fragment)(`{ address: ${programAddress}, role: AccountRole.READONLY }`).addImports(importFrom, programAddress);
    }
    return (0, common_1.fragment)('');
}
//# sourceMappingURL=instructionFunctionLowLevel.js.map