"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInstructionRemainingAccountsFragment = void 0;
const shared_1 = require("../../../shared");
const nodes_1 = require("../../../nodes");
const common_1 = require("./common");
function getInstructionRemainingAccountsFragment(scope) {
    const { remainingAccounts } = scope.instructionNode;
    const fragments = (remainingAccounts !== null && remainingAccounts !== void 0 ? remainingAccounts : []).flatMap((r) => getRemainingAccountsFragment(r, scope));
    if (fragments.length === 0)
        return (0, common_1.fragment)('');
    return (0, common_1.mergeFragments)(fragments, (r) => `// Remaining accounts.\n` +
        `const remainingAccounts: IAccountMeta[] = [...${r.join(', ...')}]`).addImports('solanaInstructions', ['IAccountMeta']);
}
exports.getInstructionRemainingAccountsFragment = getInstructionRemainingAccountsFragment;
function getRemainingAccountsFragment(remainingAccounts, scope) {
    const remainingAccountsFragment = (() => {
        if ((0, nodes_1.isNode)(remainingAccounts.value, 'argumentValueNode')) {
            return getArgumentValueNodeFragment(remainingAccounts);
        }
        if ((0, nodes_1.isNode)(remainingAccounts.value, 'resolverValueNode')) {
            return getResolverValueNodeFragment(remainingAccounts, scope);
        }
        return null;
    })();
    if (remainingAccountsFragment === null)
        return [];
    return [remainingAccountsFragment];
}
function getArgumentValueNodeFragment(remainingAccounts) {
    var _a;
    (0, nodes_1.assertIsNode)(remainingAccounts.value, 'argumentValueNode');
    const argumentName = (0, shared_1.camelCase)(remainingAccounts.value.name);
    const isWritable = (_a = remainingAccounts.isWritable) !== null && _a !== void 0 ? _a : false;
    const nonSignerRole = isWritable
        ? 'AccountRole.WRITABLE'
        : 'AccountRole.READONLY';
    const signerRole = isWritable
        ? 'AccountRole.WRITABLE_SIGNER'
        : 'AccountRole.READONLY_SIGNER';
    const role = remainingAccounts.isSigner === true ? signerRole : nonSignerRole;
    return (0, common_1.fragment)(`args.${argumentName}.map((address) => ({ address, role: ${role} }))`).addImports('solanaInstructions', ['AccountRole']);
}
function getResolverValueNodeFragment(remainingAccounts, scope) {
    var _a;
    (0, nodes_1.assertIsNode)(remainingAccounts.value, 'resolverValueNode');
    const isAsync = scope.asyncResolvers.includes(remainingAccounts.value.name);
    if (!scope.useAsync && isAsync)
        return null;
    const awaitKeyword = scope.useAsync && isAsync ? 'await ' : '';
    const functionName = scope.nameApi.resolverFunction(remainingAccounts.value.name);
    return (0, common_1.fragment)(`${awaitKeyword}${functionName}(resolverScope)`)
        .addImports((_a = remainingAccounts.value.importFrom) !== null && _a !== void 0 ? _a : 'hooked', functionName)
        .addFeatures(['instruction:resolverScopeVariable']);
}
//# sourceMappingURL=instructionRemainingAccounts.js.map